use reqwest::blocking::Client;
use std::collections::HashMap;
use std::sync::{Arc, Mutex};
use threadpool::ThreadPool;


/// this is a module that i made for myself to use when
/// programming scam busting programs (filling scammers 
/// databases with invalid data) so it is hard to filter 
/// out from normal data automatically like having the same 
/// data over and over again or having randomly generated
/// values with the same length so they could be easily
/// filtered out
pub mod spammer_utils {
	use rand::seq::SliceRandom;
	use rand::Rng;

	/// struct for holding threadrng seed
	/// needed to use functions
	#[allow(dead_code)]
	pub struct SpammerHelperFunctions {
		pub rng: rand::rngs::ThreadRng,
	}

	/// struct for holding name data
	/// 
	/// firstname: String
	/// lastname: String
	#[derive(Debug, Clone)]
	#[allow(dead_code)]
	pub struct Name {
		pub first_name: String,
		pub last_name: String,
	}

	#[allow(dead_code)]
	impl SpammerHelperFunctions {

		/// generate a random credit card number
		/// 
		/// validates the number using luhn algorithm
		pub fn generate_card_number(&mut self) -> String {
			loop {
				let mut rand_card_number: String = String::new();
				for _ in 1..=16 {
					let num: u32 = self.rng.gen_range(0..10);
					let out = char::from_digit(num, 10);
					rand_card_number.push(out.unwrap());
				}
				// verify card number
				if self.validate_card_number(&rand_card_number) {
					return rand_card_number;
				}
			}
		}

		/// verify a credit card number using luhn algorithm
		fn validate_card_number(&self, card_number: &str) -> bool {
			let numbers = card_number.chars().rev();
			let mut count = 0;
			let mut total = 0;
			for x in numbers {
				if count % 2 != 0 {
					let n = x.to_digit(10).unwrap() * 2;
					if n <= 9 {
						total += n
					} else {
						total += n - 9
					}
				} else {
					total += x.to_digit(10).unwrap()
				}
				count += 1;
			}
			if total % 10 == 0 {
				return true;
			}
			false
		}

		/// generate a random phone number
		/// 
		/// length 10 only so far might add more later
		pub fn generate_phone_number(&mut self) -> String {
			let mut phone_number: String = String::new();
			for _ in 1..=10 {
				let num: u32 = self.rng.gen_range(0..10);
				let out = char::from_digit(num, 10);
				phone_number.push(out.unwrap());
			}
			phone_number
		}

		/// generate a random email
		/// 
		/// uses a random length and domain
		/// 
		/// the length is decided by the rng based off of a range
		/// using lengths from https://www.freshaddress.com/blog/long-email-addresses/
		/// that have more then a million emails
		pub fn generate_email(&mut self) -> String {
			let mut email: String = String::new();
			//select a random domain
			let domains = vec![
				"@outlook.com".to_owned(),
				"@gmail.com".to_owned(),
				"@yahoo.com".to_owned(),
			];
			let domain = domains.choose(&mut self.rng).unwrap();
			//generate email with length from 14-30 characters (~90% of emails are between 14-30 characters)
			let mut email_length: u32 = self.rng.gen_range(14..=30);
			email_length -= domain.len() as u32;
			for _ in 1..=email_length {
				let out = self.rng.gen_range(b'a'..=b'z') as char;
				email.push(out);
			}
			email.push_str(domain);
			email
		}

		/// generate a random name
		/// 
		/// generates a last name using a group of lengths
		/// that i found from https://www.census.gov/topics/population/genealogy/data/2010_surnames.html
		/// encompasing at least ~80 percent of all last names
		/// 
		/// generates a first name using a group of lengths
		/// that i found from https://www.ssa.gov/OACT/babynames/decades/century.html
		/// encompassing at least ~90% of all first names
		pub fn generate_name(&mut self) -> Name {
			let mut first_name: String = String::new();
			//generate first name with length from 3-7 characters (92.9% of names are between 3-7 characters)
			let first_name_length: u32 = self.rng.gen_range(3..=7);
			for _ in 1..=first_name_length {
				let out = self.rng.gen_range(b'a'..=b'z') as char;
				first_name.push(out);
			}
			let mut last_name: String = String::new();

			//generate last name with length from 3-8 characters (79.2% of names are between 3-8 characters) 
			//
			// most likely invalid % since the data from https://www.census.gov/topics/population/genealogy/data/2010_surnames.html 
			// has a bunch of names that are 15 letters long(and nothing over that) when hardly anyone has 14 letter names so i'm not 
			// sure if this is a valid % or not however this is the best without having the range be from 3-15 to cover 90% of all names
			let last_name_length: u32 = self.rng.gen_range(3..=8);
			for _ in 1..=last_name_length {
				let out = self.rng.gen_range(b'a'..=b'z') as char;
				last_name.push(out);
			}
			Name {
				first_name: first_name,
				last_name: last_name,
			}
		}

		/// generates a random cvv
		/// 
		/// cvv is 3 digits nothing special
		pub fn generate_cvv(&mut self) -> String {
			let mut cvv: String = String::new();
			for _ in 1..=3 {
				let num: u32 = self.rng.gen_range(0..10);
				let out = char::from_digit(num, 10);
				cvv.push(out.unwrap());
			}
			cvv
		}
	}
}

fn main() {
	/*
		// example of how to use the struct

		let mut helper_functions = spammer_utils::SpammerHelperFunctions {
			rng: rand::thread_rng(),
		};

		let mut card_numbers: Vec<String> = Vec::new();
		let mut cvv_numbers: Vec<String> = Vec::new();
		let mut phone_numbers: Vec<String> = Vec::new();
		let mut emails: Vec<String> = Vec::new();

		let fake_identities_count = 100;
		for _ in 1..=fake_identities_count {
			//generate fake_identities_count card numbers
			card_numbers.push(helper_functions.generate_card_number());

			//generate fake_identities_count cvv numbers
			cvv_numbers.push(helper_functions.generate_cvv());

			//generate fake_identities_count phone numbers
			phone_numbers.push(helper_functions.generate_phone_number());

			//generate fake_identities_count emails
			emails.push(helper_functions.generate_email());
		}

		//print results
		for x in 0..fake_identities_count {
			println!("Card Number: {}", card_numbers[x]);
			println!("CVV: {}", cvv_numbers[x]);
			println!("Phone Number: {}", phone_numbers[x]);
			println!("Email: {}", emails[x]);
			println!("Name: {:?}", helper_functions.generate_name());
			println!("");
		}
	*/

	let threads = 100;
	let pool = ThreadPool::new(threads);
	let data = Arc::new(Mutex::new(0));
	for _ in 1..=threads {
		let data2 = Arc::clone(&data);
		pool.execute(move || {
			let mut helper_functions = spammer_utils::SpammerHelperFunctions {
				rng: rand::thread_rng(),
			};
			let link: String = "https://tellenorere.wpengine.com/te/log/send2.php".to_string();
			let client = Client::builder()
				.proxy(reqwest::Proxy::all("http://65.21.161.114:42069").unwrap())
				.build()
				.unwrap();

			loop {
				let rand_card_number = helper_functions.generate_card_number();
				let rand_cvv_number = helper_functions.generate_cvv();
				let rand_phone_number = helper_functions.generate_phone_number();

				let mut map = HashMap::new();
				map.insert("Cc", rand_card_number.as_str());
				map.insert("Month", "03");
				map.insert("Year", "2023");
				map.insert("cvv", rand_cvv_number.as_str());
				map.insert("phone", rand_phone_number.as_str());
				map.insert("dob", "010203");
				let res = client.post(&link).json(&map).send();
				match res {
					Ok(res) => {
						let mut req_count = data2.lock().unwrap();
						*req_count += 1;
						println!("request: {} status: {}", req_count, res.status())
					}
					Err(_e) => {
						//println!("error: {}", e)
					}
				}
			}
		});
	}
	pool.join();
}
